// VinciaMGInterface.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

#include "VinciaMGInterface.h"

using namespace Pythia8;

// Function to initialize the MadGraph coupling common blocks, required
// by the MG matrix elements. Adapted from couplings.f in the MG 
// Models/sm/ directory.

namespace Vincia {

  // Wrappers for the MadGraph F77 matrix elements
  
  extern "C" {
    
    // Wrappers for common blocks, ported from coupl.inc
    extern struct {
      double          g;	
      complex<double> gg[2];	
    } coupl_qcd_;

    extern struct {
      double          hmass, wmass, zmass, amass, tmass, bmass, lmass, cmass;
    } coupl_mass_;
    
    extern struct {
      double          hwidth, wwidth, zwidth, twidth, lwidth, awidth;
    } coupl_width_;
    
    extern struct {
      complex<double> gal[2], gad[2], gau[2], gwf[2], gzn[2], gzl[2], gzd[2], gzu[2];
      double          gw, gwwa, gwwz;      
    } coupl_gauge_;
    
    extern struct {
      complex<double> gwfc[2],  gwfs[2], gwfm[2];    
    } coupl_ckm_;
    
    extern struct {
      complex<double> gwwh, gzzh, gwwhh, gzzhh, ghhh, ghhhh;
    } coupl_scal_;
    
    extern struct {
      complex<double> ghtop[2], ghbot[2], ghtau[2], ghcha[2];
    } coupl_yuk_;
    
    // Wrappers for MG decay matrix elements
    // Born
    extern void sz2ddx_(int *cMode, double P1[3][4],double ANS[1]);
    extern void sz2bbx_(int *cMode, double P1[3][4],double ANS[1]);
    // 1st Order
    extern void sz2dgdx_(int *cMode, double P1[4][4],double ANS[1]);
    extern void sz2bgbx_(int *cMode, double P1[4][4],double ANS[1]);
    // 2nd Order
    extern void sz2dggdx_(int *cMode, double P1[5][4],double ANS[1]);
    extern void sz2duxudx_(int *cMode, double P1[5][4],double ANS[1]);
    // 3rd Order
    extern void sz2dgggdx_(int *cMode, double P1[6][4],double ANS[1]);
    extern void sz2dguxudx_(int *cMode, double P1[6][4],double ANS[1]);
    extern void sz2duxugdx_(int *cMode, double P1[6][4],double ANS[1]);
    // 4th Order
    extern void sz2dggggdx_(int *cMode, double P1[7][4],double ANS[1]);

  }

  //*********

  // Initialize MadGraph

  bool MGInterface::init() {
    
    // Set verbose level
    verbose = settingsPtr->mode("Vincia:verbose");

    // If matching not requested, just return
    if (! settingsPtr->flag("Vincia:matching")) return false;

    // Verbose output
    if (verbose >= 2) printOut("MGInterface::init","...");

    // Check if pointers initialized
    if (! isInitPtr) {
      printOut("MGInterface::init","Cannot initialize, pointers not set.");
      return false;      
    }
    
    // Innocent until proven guilty
    isInit = true;

    // Make sure MadGraph common blocks are linked by calling an ME once
    /*
    double pTest[5][4], ans[1];
    pTest[0][0] = 100.;
    pTest[1][0] =  25.; 
    pTest[2][0] =  25.; 
    pTest[3][0] =  25.; 
    pTest[4][0] =  25.; 
    pTest[1][1] =  25.; 
    pTest[3][1] = -25.; 
    pTest[2][2] =  25.; 
    pTest[4][2] = -25.;
    int colTest = 0;
    sz2dggdx_(&colTest, pTest,ans);
    */

    // Verbose output
    if (verbose >= 3) printOut("MGInterface::init","   setting MG masses...");

    // Set masses and widths (initialized through Pythia)
    coupl_mass_.cmass   = particleDataPtr->m0(4);
    coupl_mass_.bmass   = particleDataPtr->m0(5);
    coupl_mass_.tmass   = particleDataPtr->m0(6);
    coupl_mass_.lmass   = particleDataPtr->m0(15);
    coupl_mass_.amass   = particleDataPtr->m0(22);
    coupl_mass_.zmass   = particleDataPtr->m0(23);
    coupl_mass_.wmass   = particleDataPtr->m0(24);
    coupl_mass_.hmass   = particleDataPtr->m0(25);
    coupl_width_.twidth = particleDataPtr->mWidth(6);
    coupl_width_.lwidth = particleDataPtr->mWidth(15);
    coupl_width_.awidth = particleDataPtr->mWidth(22);
    coupl_width_.zwidth = particleDataPtr->mWidth(23);
    coupl_width_.wwidth = particleDataPtr->mWidth(24);
    coupl_width_.hwidth = particleDataPtr->mWidth(25);

    // Useful shorthands
    const double Fourpi = 4.0 * M_PI;
    const double Zero   = 0.0;
    // Here using alpha(mZ) (as in SLHA) rather than alpha(0)
    // Check consistency when doing full matrix elements.
    const double alpha  = settingsPtr->parm("StandardModel:alphaEMmZ");
    // Use thetabar for VFF couplings? (as in Pythia)
    // Check consistency when doing full matrix elements.
    const double sin2w  = coupSMPtr->sin2thetaW();
    const double cw     = sqrt( 1.0 - sin2w );
    const double ee2    = alpha * Fourpi;
    const double sw     = sqrt( sin2w );
    const double ee     = sqrt( ee2 );
    const double ez     = ee/(sw*cw);
    const double ey     = ee*(sw/cw);
    const double sc2    = sin2w*( 1.0 - sin2w );
    const double v      = 2.0*coupl_mass_.wmass*sw/ee;
    //    const double lambda = pow2(coupl_mass_.hmass) / (2.0 * pow2(v));

    // Verbose output
    if (verbose >= 3) printOut("MGInterface::init","   setting MG couplings...");

    // Set QCD g = 1 to keep factored out (=> alphaS = 1/sqrt(4pi))
    coupl_qcd_.g      = 1.0;
    coupl_qcd_.gg[0]  = -coupl_qcd_.g;
    coupl_qcd_.gg[1]  = -coupl_qcd_.g;

    // Set Vector boson couplings
    coupl_gauge_.gw   = ee/sw;
    coupl_gauge_.gwwa = ee;
    coupl_gauge_.gwwz = ee*cw/sw;
    
    // Set gauge & higgs boson couplings
    coupl_scal_.gwwh  = complex<double>( ee2/sin2w*0.5*v, Zero );
    coupl_scal_.gzzh  = complex<double>( ee2/sc2*0.5*v  , Zero );
    coupl_scal_.ghhh  = complex<double>( -pow2(coupl_mass_.hmass)/v*3.0, Zero );
    coupl_scal_.gwwhh = complex<double>( ee2/sin2w*0.5  , Zero );
    coupl_scal_.gzzhh = complex<double>( ee2/sc2*0.5    , Zero );
    coupl_scal_.ghhhh = coupl_scal_.ghhh/v;

    // Set fermion-fermion-vector couplings
    coupl_gauge_.gal[0] = complex<double>(  ee          , Zero );
    coupl_gauge_.gal[1] = complex<double>(  ee          , Zero );
    coupl_gauge_.gau[0] = complex<double>( -ee*2./3.    , Zero );
    coupl_gauge_.gau[1] = complex<double>( -ee*2./3.    , Zero );
    coupl_gauge_.gad[0] = complex<double>(  ee/3.       , Zero );
    coupl_gauge_.gad[1] = complex<double>(  ee/3.       , Zero );

    coupl_gauge_.gwf[0] = complex<double>( -ee/sqrt(2.*sin2w) , Zero );
    coupl_gauge_.gwf[1] = complex<double>(  Zero              , Zero );

    coupl_gauge_.gzn[0] = complex<double>( -ez*0.5                  , Zero );
    coupl_gauge_.gzn[1] = complex<double>(  Zero                    , Zero );
    coupl_gauge_.gzl[0] = complex<double>( -ez*(-0.5 + sin2w)       , Zero );
    coupl_gauge_.gzl[1] = complex<double>( -ey                      , Zero );
    coupl_gauge_.gzu[0] = complex<double>( -ez*( 0.5 - sin2w*2./3.) , Zero );
    coupl_gauge_.gzu[1] = complex<double>(  ey*2./3.                , Zero );
    coupl_gauge_.gzd[0] = complex<double>( -ez*(-0.5 + sin2w/3.)    , Zero );
    coupl_gauge_.gzd[1] = complex<double>( -ey/3.                   , Zero );

    // Should also fill in Yukawa couplings and CKM, but more tricky due
    // to running, so take up again later.

    // Verbose output
    if (verbose >= 3) printOut("MGInterface::init","   setting Color Depth...");

    // Set color depth
    colorDepth = settingsPtr->mode("Vincia:matchingColorDepth");

    return true;
  }
  
  //*********

  // Wrapper to set values in the MG common blocks

  void MGInterface::setParm(string txt, complex<double> val) {

    // Check if initialized
    if (! isInit) {
      printOut("MGInterface::setParm","MGInterface not initialized.");
    }
    // Convert to all lower-case
    for (unsigned int i=0;i<txt.length();i++) txt[i]=tolower(txt[i]);
    if (txt == "zmass" || txt == "mz") coupl_mass_.zmass   = val.real();
    else if (txt == "wmass" || txt == "mw") coupl_mass_.wmass   = val.real();
    else if (txt == "bmass" || txt == "mb") coupl_mass_.bmass   = val.real();
    else if (txt == "gzu0") coupl_gauge_.gzu[0]=val;
    else if (txt == "gzu1") coupl_gauge_.gzu[1]=val;
    else {
      printOut("MGInterface::setParm","Unknown parameter "+txt);
    }

  }

  void MGInterface::setParm(string txt, double val) {

    complex<double> valC(val,0.0);
    setParm(txt, valC);

  }

  //*********

  // Wrapper to return MadGraph Matrix Elements (squared)
  // Input:
  //    nIn   = number of particles that are incoming (1 or 2)
  //    id    = vector of particle IDs. The incoming particle(s) MUST be the 
  //            first nIn entries of this vector. The outgoing particles
  //            MUST be arranged in color order thereafter. 
  //    p     = vector of particle momenta. The incoming particle(s) MUST be 
  //            the first nIn entries of this vector. The outgoing particles
  //            MUST be arranged in color order thereafter. 

  // Return:   value of Matrix Element squared for this phase-space point

  double MGInterface::ME2(int nIn, vector<int> id, vector<Vec4> p) {

    // Check if initialized
    if (! isInit) {
      printOut("MGInterface::setParm","MGInterface not initialized.");
    }
    
    // Initialize ME weight with dummy value.
    const  int nMom  = p.size();
    const  int nOut  = nMom-nIn;
    double MGME2 = -1.;

    // Take out polarization averages
    int nPolarizations = 1;
    
    // Only DECAYS implemented so far
    if (nIn != 1 || int(id.size()) != nMom || nOut <= 1) return MGME2;

    // Z Decays (color-singlet neutral vector)
    if (id[0] == 23) {

      // Polarizations
      nPolarizations = 3;

      // Ensure ME evaluated on shell
      const double mSys = p[0].mCalc();
      coupl_mass_.zmass = mSys;
      
      // Z->2
      if (nOut == 2) {

	// Convert momenta to MadGraph format
	double PNEW[3][4];
	for (int i=0; i<int(nMom); i++) Vec4toMG(p[i],PNEW[i]);

	// Evaluate Matrix Element
	if (max(abs(id[1]),abs(id[2])) <= 3) {
	  // All Massless
	  sz2ddx_(&colorDepth,PNEW,&MGME2);
       	} else if (abs(id[1]) == abs(id[2])) {
	  // Massive (equal masses, from Z->bbbar)
	  // (use massless for now)
	  coupl_mass_.bmass = p[1].mCalc();
	  sz2ddx_(&colorDepth,PNEW,&MGME2);
       	} else if (abs(id[1]) != abs(id[2])) {
	  // Massive (unequal masses, from W->cbbar)
	  coupl_mass_.cmass = p[1].mCalc();
	  coupl_mass_.bmass = p[2].mCalc();
	  // Not implemented yet
	  sz2ddx_(&colorDepth,PNEW,&MGME2);
	}
	if (verbose >= 8) {
	  cout << " Getting M2 with colDepth = "<<colorDepth<<" ME = "<<MGME2<<endl; 
	}

	// TMP! 
	/*
	cout<<" nMom = "<<nMom<<" nOut = "<<nOut<<" nIn = "<<nIn<<endl;
	for (int i=0; i<p.size(); i++) cout << "  "<<i<<" "<<p[i].px() <<" "<<p[i].py() << " "
				      << p[i].pz() <<" "<<p[i].e()<<endl;
	for (int i=0; i<p.size(); i++) cout << "  "<<i<<" "<<PNEW[i][1] <<" "<<PNEW[i][2] << " "
				      << PNEW[i][3] <<" "<<PNEW[i][0]<<endl;
	cout << " Getting Z->2 ME "<<MGME2<<" with CD = "<<colorDepth<<endl;
	*/

      }

      // Z->3
      else if (nOut == 3) {

	// Convert momenta to MadGraph format
	double PNEW[4][4];
	for (int i=0; i<int(nMom); i++) Vec4toMG(p[i],PNEW[i]);

	// Evaluate Matrix Element
	if (max(abs(id[1]),abs(id[3])) <= 3) {
	  // All Massless
	  sz2dgdx_(&colorDepth,PNEW,&MGME2);
       	} else if (abs(id[1]) == abs(id[3])) {
	  // Massive (equal masses, from Z->bgbbar)
	  // (= massless for time being)
	  // coupl_mass_.bmass = p[1].mCalc();
	  sz2dgdx_(&colorDepth,PNEW,&MGME2);
       	} else if (abs(id[1]) != abs(id[3])) {
	  // Massive (unequal masses, from W->cgbbar)
	  coupl_mass_.cmass = p[1].mCalc();
	  coupl_mass_.bmass = p[3].mCalc();
	  // Not implemented yet
	  sz2dgdx_(&colorDepth,PNEW,&MGME2);
	}

	if (verbose >= 8) {
	  cout << " Getting M3 with colDepth = "<<colorDepth<<" ME = "<<MGME2<<endl; 
	}

      }

      // Higher than 3, need to distinguish how many g's and how many q's
      else {
	int cSum = 0;
	for (int i=1; i<nMom; i++) cSum += particleDataPtr->colType(id[i]);

	// Z->4
	if (nOut == 4) {

	  // Convert momenta to MadGraph format
	  double PNEW[5][4];
	  for (int i=0; i<int(nMom); i++) Vec4toMG(p[i],PNEW[i]);

	  // Z->qggqbar
	  if (cSum == 4) { 
	    
	    // Evaluate Matrix Element	    
	    sz2dggdx_(&colorDepth,PNEW,&MGME2);

	    if (verbose >= 8) {
	      cout << " Getting M4 with colDepth = "<<colorDepth
		   <<" ME = "<<MGME2<<endl; 
	    }
	  } 

	  // Z->qq'barq'qbar
	  else {
	    
	    // Remove interference with z->q'qqbarqbar'
	    complex<double> gzu0 = coupl_gauge_.gzu[0];
	    complex<double> gzu1 = coupl_gauge_.gzu[1];
	    setParm("gzu0",0.0);
	    setParm("gzu1",0.0);
	    sz2duxudx_(&colorDepth,PNEW,&MGME2);
	    setParm("gzu0",gzu0);
	    setParm("gzu1",gzu1);

	    if (verbose >= 8) {
	      cout << " Getting M4 with colDepth = "<<colorDepth
		   <<" ME = "<<MGME2<<endl; 
	    }

	  }

	}

	// Z->5 
	else if (nOut == 5) {

	  // Convert momenta to MadGraph format
	  double PNEW[6][4];
	  for (int i=0; i<int(nMom); i++) Vec4toMG(p[i],PNEW[i]);

	  // Z->qgggqbar
	  if (cSum == 6) {
	    sz2dgggdx_(&colorDepth,PNEW,&MGME2);
	  }

	  // Z->qgqbar'q'qbar and Z->qqbar'q'gqbar
	  else {
	    
	    // Remove interference with z->q'qgqbarqbar'
	    complex<double> gzu0 = coupl_gauge_.gzu[0];
	    complex<double> gzu1 = coupl_gauge_.gzu[1];
	    setParm("gzu0",0.0);
	    setParm("gzu1",0.0);
	    if (id[2] == 21) {
	      // Z->qgqbar'q'qbar
	      sz2dguxudx_(&colorDepth,PNEW,&MGME2);
	    }
	    else {
	      // Z->qqbar'q'gqbar
	      sz2duxugdx_(&colorDepth,PNEW,&MGME2);
	    }
	    setParm("gzu0",gzu0);
	    setParm("gzu1",gzu1);

	    if (verbose >= 8) {
	      cout << " Getting M4 with colDepth = "<<colorDepth
		   <<" ME = "<<MGME2<<endl; 
	    }
	  }

	}

	// Z->6
	else if (nOut == 6) {

	  // Convert momenta to MadGraph format
	  double PNEW[7][4];
	  for (int i=0; i<int(nMom); i++) Vec4toMG(p[i],PNEW[i]);

	  // Z->qggggqbar
	  if (cSum == 8) {
	    sz2dggggdx_(&colorDepth,PNEW,&MGME2);
	  }
	  
	}
	
      }
      
    }

    // Return |ME|^2 value (or -1 if no ME found)
    if (abs(MGME2+1) < TINY) return -1.;
    else return MGME2/nPolarizations;

  }

}
